/**************************************************************************//**
 * @item     CosyOS-II Hook
 * @file     init_hook.c
 * @brief    初始化钩子
 * @detail   在主函数中首先被调用，适用于初始化时钟、GPIO、寄存器等工作。
 * @author   迟凯峰
 * @version  V2.0.0
 * @date     2024.03.17
 ******************************************************************************/

#include "..\System\os_link.h"
#include "..\Test\test.h"
#include "CosyOS.h"
#include "STC32G_GPIO.h"
#include "STC32G_ADC.h"
#include "STC32G_Exti.h"
#include "STC32G_I2C.h"
#include "STC32G_SPI.h"
#include "STC32G_Timer.h"
#include "STC32G_UART.h"
#include "STC32G_Compare.h"
#include "STC32G_Delay.h"
#include "STC32G_Switch.h"
#include "STC32G_WDT.h"
#include "STC32G_NVIC.h"

/*
 * 默认配置：
 * 1、TASK_0中的printf，用串口1
 * 2、任务管理器，用串口2
 * 3、串口1、串口2，共用定时器2做波特率发生器
 * 4、串口1切换至P1.6/P1.7，串口2不切换
 * 5、PendSV_Handler，用INT0中断，中断优先级必须为最低级
 * 6、中断服务测试，会用到定时器1、定时器1中断、INT1中断、ADC中断
 *
 * 用户可根据自身情况重新调整串口
 */

void init_hook(void)
{
	COMx_InitDefine  COMx_InitStructure;  // 结构定义
	EXTI_InitTypeDef Exti_InitStructure;  // 结构定义
	#if TEST_INTSV == 1
	TIM_InitTypeDef  TIM_InitStructure;   // 结构定义
	#endif
	
	WTST = 0;   // 设置程序指令延时参数，赋值为0可将CPU执行指令的速度设置为最快
	EAXFR = 1;  // 扩展寄存器(XFR)访问使能
	CKCON = 0;  // 提高访问XRAM速度
	
////P0_MODE_IO_PU(GPIO_Pin_All);		//P0 设置为准双向口
	P1_MODE_IO_PU(GPIO_Pin_All);		//P1 设置为准双向口
	P2_MODE_IO_PU(GPIO_Pin_All);		//P2 设置为准双向口
////P3_MODE_IO_PU(GPIO_Pin_All);		//P3 设置为准双向口
////P4_MODE_IO_PU(GPIO_Pin_All);		//P4 设置为准双向口
////P5_MODE_IO_PU(GPIO_Pin_All);		//P5 设置为准双向口
////P6_MODE_IO_PU(GPIO_Pin_All);		//P6 设置为准双向口
////P7_MODE_IO_PU(GPIO_Pin_All);		//P7 设置为准双向口
	
	/* printf */
	/* UART1中断优先级为3级 */
	COMx_InitStructure.UART_Mode      = UART_8bit_BRTx;     //模式, UART_ShiftRight,UART_8bit_BRTx,UART_9bit,UART_9bit_BRTx
	COMx_InitStructure.UART_BRT_Use   = BRT_Timer2;			//选择波特率发生器, BRT_Timer1, BRT_Timer2 (注意: 串口2固定使用BRT_Timer2)
	COMx_InitStructure.UART_BaudRate  = 115200ul;			//波特率, 一般 110 ~ 115200
	COMx_InitStructure.UART_RxEnable  = DISABLE;			//接收允许,   ENABLE或DISABLE
	COMx_InitStructure.BaudRateDouble = DISABLE;			//波特率加倍, ENABLE或DISABLE
	UART_Configuration(UART1, &COMx_InitStructure);		    //初始化串口1 UART1,UART2,UART3,UART4
	NVIC_UART1_Init(ENABLE,Priority_3);		//中断使能, ENABLE/DISABLE; 优先级(低到高) Priority_0,Priority_1,Priority_2,Priority_3
	UART1_SW(UART1_SW_P16_P17);		//UART1_SW_P30_P31,UART1_SW_P36_P37,UART1_SW_P16_P17,UART1_SW_P43_P44
	
	/* 任务管理器 */
	#if SYSCFG_DEBUGGING == 1
	/* UART2中断优先级为3级 */
	COMx_InitStructure.UART_Mode      = UART_8bit_BRTx;		//模式,   UART_ShiftRight,UART_8bit_BRTx,UART_9bit,UART_9bit_BRTx
////COMx_InitStructure.UART_BRT_Use   = BRT_Timer2;			//选择波特率发生器, BRT_Timer2 (注意: 串口2固定使用BRT_Timer2, 所以不用选择)
	COMx_InitStructure.UART_BaudRate  = 115200ul;			//波特率,     110 ~ 115200
	COMx_InitStructure.UART_RxEnable  = ENABLE;				//接收允许,   ENABLE或DISABLE
	UART_Configuration(UART2, &COMx_InitStructure);		    //初始化串口2 UART1,UART2,UART3,UART4
	NVIC_UART2_Init(ENABLE,Priority_3);		//中断使能, ENABLE/DISABLE; 优先级(低到高) Priority_0,Priority_1,Priority_2,Priority_3
	UART2_SW(UART2_SW_P10_P11);		//UART2_SW_P10_P11,UART2_SW_P46_P47
	#endif
	
	/* PendSV_Handler */
	/* INT0中断优先级为0级 */
	Exti_InitStructure.EXTI_Mode = EXT_MODE_Fall;      //中断模式,   EXT_MODE_RiseFall,EXT_MODE_Fall
	Ext_Inilize(EXT_INT0,&Exti_InitStructure);				//初始化
	NVIC_INT0_Init(ENABLE,Priority_0);		//中断使能, ENABLE/DISABLE; 优先级(低到高) Priority_0,Priority_1,Priority_2,Priority_3
	
	/* 中断服务测试 */
	#if TEST_INTSV == 1
	/* INT1中断优先级为1级 */
	Exti_InitStructure.EXTI_Mode = EXT_MODE_Fall;      //中断模式,   EXT_MODE_RiseFall,EXT_MODE_Fall
	Ext_Inilize(EXT_INT1,&Exti_InitStructure);				//初始化
	NVIC_INT1_Init(ENABLE,Priority_1);		//中断使能, ENABLE/DISABLE; 优先级(低到高) Priority_0,Priority_1,Priority_2,Priority_3
	/* ADC中断优先级为2级 */
	NVIC_ADC_Init(ENABLE,Priority_2);
	/* 定时器1中断优先级为3级 */
	TIM_InitStructure.TIM_Mode      = TIM_16BitAutoReload;	//指定工作模式,   TIM_16BitAutoReload,TIM_16Bit,TIM_8BitAutoReload,TIM_16BitAutoReloadNoMask
	TIM_InitStructure.TIM_ClkSource = TIM_CLOCK_1T;		//指定时钟源, TIM_CLOCK_1T,TIM_CLOCK_12T,TIM_CLOCK_Ext
	TIM_InitStructure.TIM_ClkOut    = DISABLE;				//是否输出高速脉冲, ENABLE或DISABLE
////TIM_InitStructure.TIM_Value     = 65536UL - (MAIN_Fosc / (1000000UL / 50));
	timer1_init();
	TIM_InitStructure.TIM_Run       = ENABLE;					//是否初始化后启动定时器, ENABLE或DISABLE
	Timer_Inilize(Timer1,&TIM_InitStructure);					//初始化Timer1	  Timer0,Timer1,Timer2,Timer3,Timer4
	NVIC_Timer1_Init(ENABLE,Priority_3);		//中断使能, ENABLE/DISABLE; 优先级(低到高) Priority_0,Priority_1,Priority_2,Priority_3
	#endif
}



/*
 * UART2中断（中断优先级为3级，using bank3）
 */

#if SYSCFG_DEBUGGING == 1
void UART2_Isr(void) interrupt 8 using 3
{
	if(S2TI){
		S2TI = 0;
		if(*s_debug_sendptr){
			S2BUF = *s_debug_sendptr++;
		}
		else{
			iDebugSend;
		}
	}
	if(S2RI){
		S2RI = 0;
		if(s_debug_recvptr - s_debug_recvbuff < OS_DEBUGRECVBUFFSIZE - 1){
			*s_debug_recvptr++ = S2BUF;
		}
		iDebugRecv;
	}
}
#endif
